Package: metaseqR2
Type: Package
Title: An R package for the analysis and result reporting of RNA-Seq
        data by combining multiple statistical algorithms
Authors@R: c(person(given="Panagiotis", family="Moulos",
    email="moulos@fleming.gr", role=c("aut", "cre")))
Depends: R (>= 4.0.0), DESeq2, limma, locfit, splines
Imports: ABSSeq, Biobase, BiocGenerics, BiocParallel, biomaRt,
        Biostrings, corrplot, DSS, DT, EDASeq, edgeR, genefilter,
        Seqinfo, GenomeInfoDb, GenomicAlignments, GenomicFeatures,
        GenomicRanges, gplots, graphics, grDevices, harmonicmeanp,
        heatmaply, htmltools, httr, IRanges, jsonlite, lattice, log4r,
        magrittr, MASS, Matrix, methods, NBPSeq, pander, parallel,
        qvalue, rmarkdown, rmdformats, Rsamtools, RSQLite, rtracklayer,
        S4Vectors, stats, stringr, SummarizedExperiment, survcomp,
        txdbmaker, utils, VennDiagram, vsn, yaml, zoo
Suggests: BiocStyle, BiocManager, BSgenome, knitr, RMySQL, RUnit
Enhances: TCC
Description: Provides an interface to several normalization and
    statistical testing packages for RNA-Seq gene expression data.
    Additionally, it creates several diagnostic plots, performs
    meta-analysis by combinining the results of several statistical
    tests and reports the results in an interactive way.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: false
NeedsCompilation: yes
URL: http://www.fleming.gr
biocViews: Software, GeneExpression, DifferentialExpression,
        WorkflowStep, Preprocessing, QualityControl, Normalization,
        ReportWriting, RNASeq, Transcription, Sequencing,
        Transcriptomics, Bayesian, Clustering, CellBiology,
        BiomedicalInformatics, FunctionalGenomics, SystemsBiology,
        ImmunoOncology, AlternativeSplicing, DifferentialSplicing,
        MultipleComparison, TimeCourse, DataImport, ATACSeq,
        Epigenetics, Regression, ProprietaryPlatforms,
        GeneSetEnrichment, BatchEffect, ChIPSeq
VignetteBuilder: knitr
BugReports: https://github.com/pmoulos/metaseqR2/issues
Version: 1.25.0
Date: 2026-03-21
Collate: 'annotation.R' 'argcheck.R' 'count.R' 'deseq.R' 'export.R'
        'external.R' 'filter.R' 'json.R' 'main.R' 'meta.R' 'noiseq.R'
        'norm.R' 'metaseqr2-data.R' 'metaseqR2-package.R' 'plot.R'
        'query.R' 'reportdb.R' 'sim.R' 'stat.R' 'tracks.R' 'util.R'
        'zzz.R'
git_url: https://git.bioconductor.org/packages/metaseqR2
git_branch: devel
git_last_commit: 0bd17c4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
Packaged: 2026-04-29 22:52:02 UTC; biocbuild
Author: Panagiotis Moulos [aut, cre]
Maintainer: Panagiotis Moulos <moulos@fleming.gr>
