The files C.bam and T.bam are also included in the Bioconductor package recoup
and are part of the data described in Nikolaou et al., 2017, PMID: 28746875 
(https://www.ncbi.nlm.nih.gov/pubmed/28746875)
Specifically, each file comprises 50000 random reads from H4K20me3 methylation
data in wild-type adult mice (C.bam) and the respective knock-out of the Kmt5a 
gene (T.bam).
These are not RNA-Seq data but are well suited to demonstrate the counting and
other functions they are used with.