#' An R package for the analysis and result reporting of RNA-Seq gene expression
#' data, using multiple statistical algorithms.
#'
#' \tabular{ll}{
#' Package: \tab metaseqR2\cr
#' Type: \tab Package\cr
#' Version: \tab 0.0.20\cr
#' Date: \tab 2020-03-13\cr
#' Depends: \tab R (>= 4.0.0), limma, qvalue\cr
#' Encoding: \tab UTF-8\cr
#' License: \tab GPL (>= 3)\cr
#' LazyLoad: \tab yes\cr
#' URL: \tab http://www.fleming.gr\cr
#' }
#'
#' Provides an interface to several normalization and statistical testing 
#' packages for RNA-Seq gene expression data. Additionally, it creates several
#' diagnostic plots, performs meta-analysis by combinining the results of 
#' several statistical tests and reports the results in an interactive way.
#'
#' @aliases metaseqR2-package metaseqR2
#' @name metaseqR2-package
#' @docType package
#' @title The metaseqR2 Package
#' @author Panagiotis Moulos \email{moulos@@fleming.gr}
#' @keywords package
NULL
